@@ -43,7 +43,7 @@
                               xkf, nkqtotf, esigmar_all, esigmai_all, a_all, nbndfst
     USE ep_constants,  ONLY : kelvin2eV, ryd2mev, one, ryd2ev, two, zero, pi
     USE mp,            ONLY : mp_sum
-    USE mp_global,     ONLY : inter_pool_comm
+    USE mp_global,     ONLY : inter_pool_comm, inter_image_comm
     USE parallelism,   ONLY : poolgather2
     !
     IMPLICIT NONE
@@ -113,6 +113,8 @@
     CALL poolgather2(nbndsub, nkqtotf, nkqf, etf, etf_all)
     CALL mp_sum(esigmar_all, inter_pool_comm)
     CALL mp_sum(esigmai_all, inter_pool_comm)
+    CALL mp_sum(esigmar_all, inter_image_comm)
+    CALL mp_sum(esigmai_all, inter_image_comm)
     !
     DO itemp = 1, nstemp ! second temperature loop to write data
       !
@@ -232,8 +234,8 @@
     DEALLOCATE(etf_all, STAT = ierr)
     IF (ierr /= 0) CALL errore('spectral_func_el_q', 'Error deallocating etf_all', 1)
     !
-    101 FORMAT(5x, 'ik = ', i7, '  w = ', f9.4, ' eV   A(k,w) = ', e12.5, ' meV^-1')
-    102 FORMAT(2i9, 2x, f12.4, 2x, f12.4, 2x, f12.4, 2x, f12.4, 2x, f12.4)
+    101 FORMAT(5x, 'ik = ', i7, '  w = ', f9.4, ' eV   A(k,w) = ', ES13.5, ' meV^-1')
+    102 FORMAT(2i9, 2x, ES13.5, 2x, ES13.5, 2x, ES13.5, 2x, ES13.5, 2x, ES13.5)
     !
     RETURN
     !
@@ -260,17 +262,20 @@
     USE io_global, ONLY : stdout, ionode
     USE io_var,    ONLY : iospectral_sup, iospectral
     USE modes,     ONLY : nmodes
-    USE input,     ONLY : nbndsub, fsthick, shortrange, ngaussw, degaussw, &
-                          nsmear, eps_acoustic, efermi_read, fermi_energy, &
-                           nstemp, wmin_specfun, wmax_specfun, nw_specfun
+    USE input,     ONLY : fsthick, shortrange, ngaussw, degaussw, &
+                          nsmear, delta_smear, eps_acoustic, nstemp, &
+                          wmin_specfun, wmax_specfun, nw_specfun, &
+                          phonselfen
     USE pwcom,     ONLY : nelec, ef
     USE input,     ONLY : isk_dummy
     USE global_var,ONLY : gtemp, epf17, ibndmin, etf, nbndfst, &
-                          wkf, xqf, nkqf, nkf, wf, a_all_ph, efnew
+                          wkf, xqf, nkqf, nkf, wf, a_all_ph, efnew, pi_0, &
+                          pir_all, gammai_all
     USE ep_constants,  ONLY : kelvin2eV, ryd2mev, ryd2ev, one, two, zero, cone, ci, eps8
     USE constants,     ONLY : pi
     USE mp,            ONLY : mp_barrier, mp_sum
     USE mp_global,     ONLY : inter_pool_comm
+    USE selfen,        ONLY : selfen_phon_q
     !
     IMPLICIT NONE
     !
@@ -280,28 +285,19 @@
     !! File name for spectral function
     CHARACTER(LEN = 256) :: filespecsup
     !! File name for supporting information
+    !
     INTEGER, INTENT(in) :: iqq
     !! Current q-point index from selecq
     INTEGER, INTENT(in) :: iq
     !! Current q-point index
     INTEGER, INTENT(in) :: totq
     !! Total q-points in selecq window
+    INTEGER :: ismear
+    !! Number of smearing values for the Gaussian function
     !
     ! Local variables
-    INTEGER :: ik
-    !! Counter on the k-point index
-    INTEGER :: ikk
-    !! k-point index
-    INTEGER :: ikq
-    !! q-point index
-    INTEGER :: ibnd
-    !! Counter on bands
-    INTEGER :: jbnd
-    !! Counter on bands
     INTEGER :: imode
     !! Counter on mode
-    INTEGER :: fermicount
-    !! Number of states on the Fermi surface
     INTEGER :: iw
     !! Counter on frequency for the phonon spectra
     INTEGER :: itemp
@@ -311,45 +307,10 @@
     !
     REAL(KIND = DP) :: g2
     !! Electron-phonon matrix elements squared in Ry^2
-    REAL(KIND = DP) :: ef0
-    !! Fermi energy level
-    REAL(KIND = DP) :: dosef
-    !! Density of state N(Ef)
-    REAL(KIND = DP) :: ekk
-    !! Eigen energy on the fine grid relative to the Fermi level
-    REAL(KIND = DP) :: ekq
-    !! Eigen energy of k+q on the fine grid relative to the Fermi level
-    REAL(KIND = DP) :: wgkk
-    !! Fermi-Dirac occupation factor $f_{nk}(T)$
-    REAL(KIND = DP) :: wgkq
-    !! Fermi-Dirac occupation factor $f_{nk+q}(T)$
-    REAL(KIND = DP) :: fact
-    !! Temporary variable to store fact = wgkq - wgkk
-    REAL(KIND = DP) :: etmp1
-    !! Temporary variable to store etmp1 = ekq - ekk
-    REAL(KIND = DP) :: etmp2
-    !! Temporary variable to store etmp2 = ekq - ekk - ww
-    REAL(KIND = DP) :: weight
-    !! Self-energy factor
-    REAL(KIND = DP) :: inv_degaussw
-    !! Inverse Gaussian for efficiency reasons
-    REAL(KIND = DP) :: inv_eptemp
-    !! Inverse temperature
     REAL(KIND = DP) :: inv_pi
     !! Inverse pi
     REAL(KIND = DP) :: dw
     !! Frequency intervals
-    REAL(KIND = DP), EXTERNAL :: dos_ef
-    !! Function to compute the Density of States at the Fermi level
-    REAL(KIND = DP), EXTERNAL :: efermig
-    !! Function to compute the Fermi energy
-    REAL(KIND = DP), EXTERNAL :: wgauss
-    !! Fermi-Dirac distribution function (when -99)
-    REAL(KIND = DP), EXTERNAL :: w0gauss
-    !! This function computes the derivative of the Fermi-Dirac function
-    !! It is therefore an approximation for a delta function
-    REAL(KIND = DP) :: gamma0(nmodes)
-   !! Phonon self-energy
     REAL(KIND = DP) :: wq(nmodes)
     !! Phonon frequency on the fine grid
     REAL(KIND = DP) :: inv_wq(nmodes)
@@ -360,16 +321,20 @@
     !! If the phonon frequency is too small discart g
     REAL(KIND = DP) :: ww(nw_specfun)
     !! Current frequency
-    REAL(KIND = DP) :: gammai_all(nw_specfun, nmodes)
-    !! Imaginary part of the frequency dependent spectral function
-    REAL(KIND = DP) :: gammar_all(nw_specfun, nmodes)
-    !!  Real part of the Phonon self-energy (freq. dependent for spectral function)
+    REAL(KIND = DP) :: degaussw0
+    !! degaussw0 = (ismear-1) * delta_smear + degaussw
+    REAL(KIND = DP) :: eta
+    !! artificial broadening of 0.1 meV for the visualization of the spectral function
+    !
     !
     dw = (wmax_specfun - wmin_specfun) / DBLE(nw_specfun - 1)
     DO iw = 1, nw_specfun
       ww(iw) = wmin_specfun + DBLE(iw - 1) * dw
     ENDDO
     !
+    eta = 0.1d0 / ryd2mev
+    !
+    !
     ! Now pre-treat phonon modes for efficiency
     ! Treat phonon frequency and Bose occupation
     wq(:)    = zero
@@ -378,18 +343,25 @@
       wq(imode) = wf(imode, iq)
       dwq(imode) = two * wq(imode)
       IF (wq(imode) > eps_acoustic) THEN
-        g2_tmp(imode) = one
         inv_wq(imode) = one / (two * wq(imode))
       ELSE
-        g2_tmp(imode) = zero
         inv_wq(imode) = zero
       ENDIF
     ENDDO
     !
+    !
+    IF (.NOT. phonselfen) THEN
+      CALL selfen_phon_q(iqq, iq, totq)
+    ENDIF
+    !
+    ! collect contributions from all pools (sum over k-points) this finishes the integral over the BZ  (k)
+    !
+    CALL mp_sum(gammai_all, inter_pool_comm)
+    CALL mp_sum(pir_all, inter_pool_comm)
+    CALL mp_barrier(inter_pool_comm)
+    !
+    !
     DO itemp = 1, nstemp
-      gammar_all(:, :) = zero
-      gammai_all(:, :) = zero
-      !
       ! Thomas-Fermi screening according to Resta PRB 1977
       ! Here specific case of Diamond
       !eps0   = 5.7
@@ -407,184 +379,82 @@
         WRITE(stdout, '(/5x, a, f10.6, a)' ) 'Golden Rule strictly enforced with T = ', gtemp(itemp) * ryd2ev, ' eV'
       ENDIF
       !
-      ! SP: Multiplication is faster than division ==> Important if called a lot in inner loops
-      inv_degaussw = one / degaussw
-      inv_eptemp   = one / gtemp(itemp)
-      inv_pi       = one / pi
-      !
-      ! Fermi level and corresponding DOS
-      !
-      IF (efermi_read) THEN
+      DO ismear = 1, nsmear
         !
-        ef0 = fermi_energy
+        degaussw0 = (ismear - 1) * delta_smear + degaussw
         !
-      ELSEIF (nsmear > 1) THEN
+        ! SP: Multiplication is faster than division ==> Important if called a lot
+        !     in inner loops
+        inv_pi       = one / pi
         !
-        ! if some bands are skipped (nbndskip /= 0), nelec has already been recalculated in ephwann_shuffle
-        ef0 = efermig(etf, nbndsub, nkqf, nelec, wkf, degaussw, ngaussw, 0, isk_dummy)
+        WRITE(stdout, '(5x, a)')
         !
-      ELSE !SP: This is added for efficiency reason because the efermig routine is slow
-        ef0 = efnew
-      ENDIF
-      !
-      ! N(Ef) in the equation for lambda is the DOS per spin
-      dosef = dos_ef(ngaussw, degaussw, ef0, etf, wkf, nkqf, nbndsub)
-      dosef = dosef / two
-      !
-      IF (iqq == 1) THEN
-        WRITE(stdout, 100) degaussw * ryd2ev, ngaussw
-        WRITE(stdout, 101) dosef / ryd2ev, ef0 * ryd2ev
-      ENDIF
-      !
-      CALL start_clock('PH SPECTRAL-FUNCTION')
-      !
-      fermicount = 0
-      gamma0(:)  = zero
-      !
-      DO ik = 1, nkf
-        ikk = 2 * ik - 1
-        ikq = ikk + 1
+        IF (iqq == 1 .and. itemp == 1) THEN
+          IF (ionode) THEN
+            WRITE(tp, "(f8.1)") gtemp(itemp) * ryd2ev / kelvin2eV
+            filespecsup = 'specfun_sup.phon'! // trim(adjustl(tp)) // 'K'
+            OPEN(UNIT = iospectral_sup, FILE = filespecsup)
+            WRITE(iospectral_sup, '(2x, a)') '#Phonon eigenenergies + real and im part of phonon self-energy (meV)'
+            WRITE(iospectral_sup, '(2x, a)') '#Q-point    Mode      Temp.[K]       smearing[eV]       w_q[eV]    &    
+                                          &w[eV]     Real Pi(w, T_low)[meV]   Real Pi(w=0,T_high)[meV]     Im Pi(w, T_low)[meV]'
+          ENDIF
+        ENDIF
         !
-        ! Here we must have ef, not ef0, to be consistent with ephwann_shuffle
-        IF ((MINVAL(ABS(etf(:, ikk) - ef)) < fsthick) .AND. &
-            (MINVAL(ABS(etf(:, ikq) - ef)) < fsthick)) THEN
-          !
-          fermicount = fermicount + 1
+        ! Write to output file
+        WRITE(stdout, '(/5x, a)') 'Real and Imaginary part of the phonon self-energy (omega=0) without gamma0.'
+        DO imode = 1, nmodes
+          ! Real and Im part of Phonon self-energy at 0 freq.
+          WRITE(stdout, 105) imode, ryd2ev * wq(imode), ryd2mev * pir_all(1, imode, itemp, ismear), &
+                             ryd2mev * gammai_all(1, imode, itemp, ismear)
+        ENDDO
+        !
+        ! Write to support files
+        DO iw = 1, nw_specfun
           !
           DO imode = 1, nmodes
             !
-            DO ibnd = 1, nbndfst
-              !
-              !  the fermi occupation for k
-              ekk = etf(ibndmin - 1 + ibnd, ikk) - ef0
-              wgkk = wgauss(-ekk * inv_eptemp, -99)
-              !
-              DO jbnd = 1, nbndfst
-                !
-                !  the fermi occupation for k+q
-                ekq = etf(ibndmin - 1 + jbnd, ikq) - ef0
-                wgkq = wgauss(-ekq * inv_eptemp, -99)
-                !
-                ! here we take into account the zero-point DSQRT(hbar/2M\omega)
-                ! with hbar = 1 and M already contained in the eigenmodes
-                ! g2 is Ry^2, wkf must already account for the spin factor
-                !
-                IF (shortrange .AND. (ABS(xqf(1, iq))> eps8 .OR. ABS(xqf(2, iq))> eps8 &
-                   .OR. ABS(xqf(3, iq))> eps8 )) THEN
-                  ! SP: The abs has to be removed. Indeed the epf17 can be a pure imaginary
-                  !     number, in which case its square will be a negative number.
-                  g2 = REAL((epf17(jbnd, ibnd, imode, ik)**two) * inv_wq(imode) * g2_tmp(imode)) !* epsTF
-                ELSE
-                  g2 = (ABS(epf17(jbnd, ibnd, imode, ik))**two) * inv_wq(imode) * g2_tmp(imode) !* epsTF
-                ENDIF
-                !
-                ! SP - 03/2019 - Retarded phonon self-energy
-                !                See Eq. 145 of RMP 89, 015003 (2017)
-                ! \Pi^R = k-point weight * [ [f(E_k+q) - f(E_k)]/ [E_k+q - E_k -w_q - id]
-                !                           -[f(E_k+q) - f(E_k)]/ [E_k+q - E_k - id] ]
-                ! The second term is gamma0 (static)
-                !
-                fact = wgkq - wgkk
-                etmp1 = ekq - ekk
-                weight = wkf(ikk) * fact * REAL(cone / (etmp1 + ci * degaussw))
-                !
-                gamma0(imode) = gamma0(imode) + weight * g2
-                !
-                DO iw = 1, nw_specfun
-                  !
-                  etmp2 = etmp1 - ww(iw)
-                  weight = wkf(ikk) * fact * REAL(cone / (etmp2 + ci * degaussw))
-                  gammar_all(iw, imode) = gammar_all(iw, imode) + weight * g2
-                  !
-                  ! Normal implementation
-                  !weight = wkf (ikk) * fact * AIMAG(cone / (etmp2 + ci * degaussw))
-                  !
-                  ! More stable:
-                  ! Analytical im. part
-                  weight = pi * wkf(ikk) * fact * w0gauss(etmp2 * inv_degaussw, 0) * inv_degaussw
-                  !
-                  gammai_all(iw, imode) = gammai_all(iw, imode) + weight * g2
-                  !
-                ENDDO
-              ENDDO ! jbnd
-            ENDDO   ! ibnd
-          ENDDO ! loop on q-modes
-        ENDIF ! endif fsthick
-      ENDDO ! loop on k
-      !
-      CALL stop_clock('PH SPECTRAL-FUNCTION')
-      !
-#if defined(__MPI)
-      !
-      ! collect contributions from all pools (sum over k-points) this finishes the integral over the BZ  (k)
-      !
-      CALL mp_sum(gammai_all, inter_pool_comm)
-      CALL mp_sum(gammar_all, inter_pool_comm)
-      CALL mp_sum(gamma0, inter_pool_comm)
-      CALL mp_sum(fermicount, inter_pool_comm)
-      CALL mp_barrier(inter_pool_comm)
-      !
-#endif
-      !
-      WRITE(stdout, '(5x, a)')
-      !
-      IF (iqq == 1 .and. itemp == 1) THEN
-        IF (ionode) THEN
-          WRITE(tp, "(f8.1)") gtemp(itemp) * ryd2ev / kelvin2eV
-          filespecsup = 'specfun_sup.phon'! // trim(adjustl(tp)) // 'K'
-          OPEN(UNIT = iospectral_sup, FILE = filespecsup)
-          WRITE(iospectral_sup, '(2x, a)') '#Phonon eigenenergies + real and im part of phonon self-energy (meV)'
-          WRITE(iospectral_sup, '(2x, a)') '#Q-point    Mode      Temp.[K]       w_q[eV]        w[eV]    &
-                                           &Real Sigma(w)[meV]   Real Sigma(w=0)[meV]     Im Sigma(w)[meV]'
-        ENDIF
-      ENDIF
-      !
-      ! Write to output file
-      WRITE(stdout, '(/5x, a)') 'Real and Imaginary part of the phonon self-energy (omega=0) without gamma0.'
-      DO imode = 1, nmodes
-        ! Real and Im part of Phonon self-energy at 0 freq.
-        WRITE(stdout, 105) imode, ryd2ev * wq(imode), ryd2mev * gammar_all(1, imode), ryd2mev * gammai_all(1, imode)
-      ENDDO
-      WRITE(stdout, '(5x, a, i8, a, i8)' ) 'Number of (k,k+q) pairs on the Fermi surface: ', fermicount, ' out of ', totq
-      !
-      ! Write to support files
-      DO iw = 1, nw_specfun
-        !
-        DO imode = 1, nmodes
+            !a_all(iw,iq) = a_all(iw,iq) + ABS(gammai_all(imode,iq,iw) ) / pi / &
+            !      ( ( ww - wq - pir_all (imode,iq,iw) + pi_0 (imode))**two + (gammai_all(imode,iq,iw) )**two )
+            ! SP: From Eq. 16 of PRB 9, 4733 (1974)
+            !    Also in Eq.2 of PRL 119, 017001 (2017).
+            ! in this spectral function, phonons are unscreened with the static part calculated at the high smearing values from the DFPT calculation
+            a_all_ph(iw, iqq, itemp, ismear) = a_all_ph(iw, iqq, itemp, ismear) + inv_pi * dwq(imode) * &
+                                    (two * ww(iw) * eta - dwq(imode) * gammai_all(iw, imode, itemp, ismear)) / &
+                                    ((ww(iw)**two - eta**two - wq(imode)**two - dwq(imode) * &
+                                    (pir_all(iw, imode, itemp, ismear) - pi_0(imode)))**two + (two*ww(iw) * eta - dwq(imode) * &
+                                    gammai_all(iw, imode, itemp, ismear))**two)
+            !
+            IF (ionode) THEN
+              WRITE(iospectral_sup, 102) iq, imode, gtemp(itemp) * ryd2ev / kelvin2eV, degaussw0 * ryd2ev, ryd2ev * wq(imode), &
+                                       ryd2ev * ww(iw), ryd2mev * pir_all(iw, imode, itemp, ismear), ryd2mev * pi_0(imode), &
+                                       ryd2mev * gammai_all(iw, imode, itemp, ismear)
+            ENDIF
+            !
+          ENDDO
           !
-          !a_all(iw,iq) = a_all(iw,iq) + ABS(gammai_all(imode,iq,iw) ) / pi / &
-          !      ( ( ww - wq - gammar_all (imode,iq,iw) + gamma0 (imode))**two + (gammai_all(imode,iq,iw) )**two )
-          ! SP: From Eq. 16 of PRB 9, 4733 (1974)
-          !    Also in Eq.2 of PRL 119, 017001 (2017).
-          a_all_ph(iw, iqq, itemp) = a_all_ph(iw, iqq, itemp) + inv_pi * dwq(imode)**two * ABS(gammai_all(iw, imode)) / &
-                              ((ww(iw)**two - wq(imode)**two - dwq(imode) * (gammar_all(iw, imode) - gamma0(imode)))**two + &
-                               (dwq(imode) * gammai_all(iw, imode))**two)
+!         IF (ionode) THEN
+!           WRITE(iospectral, 103) iq, ryd2ev * ww(iw), a_all_ph(iw, iqq) / ryd2mev ! print to file
+!         ENDIF
           !
-          IF (ionode) THEN
-            WRITE(iospectral_sup, 102) iq, imode, gtemp(itemp) * ryd2ev / kelvin2eV, ryd2ev * wq(imode), ryd2ev * ww(iw), &
-                                       ryd2mev * gammar_all(iw, imode), ryd2mev * gamma0(imode), ryd2mev * gammai_all(iw, imode)
-          ENDIF
-          !
-        ENDDO
-        !
-!        IF (ionode) THEN
-!          WRITE(iospectral, 103) iq, ryd2ev * ww(iw), a_all_ph(iw, iqq) / ryd2mev ! print to file
-!        ENDIF
-        !
-      ENDDO
+        ENDDO !iw
+      ENDDO !ismear
     ENDDO ! itemp
-      !
+    !
     IF (iqq == totq) THEN
       IF (ionode) THEN
         DO itemp = 1, nstemp
           WRITE(tp, "(f8.3)") gtemp(itemp) * ryd2ev / kelvin2eV
           filespec = 'specfun.phon.' // trim(adjustl(tp)) // 'K'
-          OPEN(UNIT = iospectral, FILE = filespec)
+          OPEN(UNIT = iospectral, FILE = filespec)                
           WRITE(iospectral, '(/2x, a)') '#Phonon spectral function (meV)'
-          WRITE(iospectral, '(/2x, a)') '#Q-point    Energy[eV]     A(q,w)[meV^-1]'
-          DO iqq_write = 1, totq
-            DO iw = 1, nw_specfun
-              WRITE(iospectral, 103) iqq_write, ryd2ev * ww(iw), a_all_ph(iw, iqq_write, itemp) / ryd2mev ! print to file
+          WRITE(iospectral, '(/2x, a)') '#Q-point    Energy[eV]      smearing[eV]     A(q,w)[meV^-1]'
+          DO ismear = 1, nsmear
+            degaussw0 = (ismear - 1) * delta_smear + degaussw
+            DO iqq_write = 1, totq
+              DO iw = 1, nw_specfun
+                WRITE(iospectral, 103) iqq_write, ryd2ev * ww(iw), degaussw0 * ryd2ev, &
+                                       a_all_ph(iw, iqq_write, itemp, ismear) / ryd2mev ! print to file
+              ENDDO
             ENDDO
           ENDDO
           CLOSE(iospectral)
@@ -596,8 +466,8 @@
     !
     100 FORMAT(5x, 'Gaussian Broadening: ', f10.6, ' eV, ngauss=', i4)
     101 FORMAT(5x, 'DOS =', f10.6,' states/spin/eV/Unit Cell at Ef=', f10.6, ' eV')
-    102 FORMAT(2i9, 2x, f8.3, 2x, f12.5, 2x, f12.5, 2x, E22.14, 2x, E22.14, 2x, E22.14)
-    103 FORMAT(2x, i7, 2x, f12.5, 2x, E22.14)
+    102 FORMAT(2i9, 2x, f8.3, 2x, f12.5, 2x, f12.5, 2x, f12.5, 2x, E22.14, 2x, E22.14, 2x, E22.14)
+    103 FORMAT(2x, i7, 2x, f12.5, 2x, f12.5, 2x, E22.14)
     105 FORMAT(5x, 'Omega( ', i3, ' )=', f9.4,' eV   Re[Pi]=', f15.6, ' meV Im[Pi]=', f15.6, ' meV')
     !
     RETURN
