@@ -608,7 +608,6 @@ subroutine read_export (pp_file,kunit,uspp_spsi, ascii, single_file, raw)
   use mp,             ONLY : mp_sum, mp_max
   use mp_world,       ONLY : world_comm, nproc, mpime
   use ldaU,           ONLY : lda_plus_u
-  USE basis,          ONLY : swfcatom
   USE uspp_init,      ONLY : init_us_2
 
   implicit none
@@ -776,7 +775,8 @@ subroutine read_export (pp_file,kunit,uspp_spsi, ascii, single_file, raw)
      IF( (ik >= iks) .AND. (ik <= ike) ) THEN
 
        call davcio (evc, 2*nwordwfc, iunwfc, (ik-iks+1), - 1)
-       IF ( lda_plus_u ) CALL davcio( swfcatom, nwordatwfc, iunsat, (ik-iks+1), -1 )
+       ! IF ( lda_plus_u ) CALL davcio( swfcatom, nwordatwfc, iunsat, (ik-iks+1), -1 )
+       IF ( lda_plus_u ) call errore('pw4gww','DFT+U possibly unsupported',1)
        local_pw = ngk(ik-iks+1)
 
      ENDIF
