@@ -25,8 +25,8 @@ subroutine hp_allocate_q
   USE lrus,                 ONLY : becp1
   USE eqv,                  ONLY : dpsi, evq, dmuxc, dvpsi
   USE control_lr,           ONLY : lgamma
-  USE ldaU,                 ONLY : Hubbard_lmax, nwfcU
-  USE ldaU_lr,              ONLY : swfcatomk, swfcatomkpq
+  USE ldaU,                 ONLY : Hubbard_lmax, nwfcU, lda_plus_u_kind, max_num_neighbors
+  USE ldaU_lr,              ONLY : swfcatomk, swfcatomkpq, vh_u_save, vh_uv_save
   USE qpoint_aux,           ONLY : becpt
   USE lr_nc_mag,            ONLY : deeq_nc_save
   USE uspp_param,           ONLY : nhm 
@@ -70,6 +70,12 @@ subroutine hp_allocate_q
      ALLOCATE (swfcatomkpq(npwx*npol,nwfcU))
   ENDIF
   !
+  IF (lda_plus_u_kind == 0) THEN
+     ALLOCATE (vh_u_save(2*Hubbard_lmax+1, 2*Hubbard_lmax+1, nspin, nat, 2))
+  ELSEIF (lda_plus_u_kind == 2) THEN
+     ALLOCATE (vh_uv_save(2*Hubbard_lmax+1, 2*Hubbard_lmax+1, max_num_neighbors, nat, nspin, 2))
+  ENDIF
+  !
   RETURN
   !
 end subroutine hp_allocate_q
