@@ -27,6 +27,7 @@ subroutine dvqpsi_us (ik, uact, addnlcc, becp1, alphap)
   USE fft_interfaces,   ONLY : fwfft, invfft
   USE lsda_mod,         ONLY : lsda, isk, current_spin
   USE noncollin_module, ONLY : npol
+  USE ldaU,             ONLY : lda_plus_u
   USE wvfct,            ONLY : nbnd, npwx
   USE wavefunctions,    ONLY : evc
   USE eqv,              ONLY : dvpsi
@@ -34,6 +35,8 @@ subroutine dvqpsi_us (ik, uact, addnlcc, becp1, alphap)
   USE klist,            ONLY : ngk, igk_k
   USE qpoint,           ONLY : nksq
   USE becmod,           ONLY : bec_type
+  USE gvect,            ONLY : gg
+  USE control_lr,       ONLY : lmultipole
   !
   IMPLICIT NONE
   !
@@ -100,8 +103,23 @@ subroutine dvqpsi_us (ik, uact, addnlcc, becp1, alphap)
   !
   ! Compute dV_loc/dtau in real space
   !
-  CALL compute_dvloc (uact, addnlcc, dvlocin)
-  !
+  IF (.NOT. lmultipole) THEN
+    CALL compute_dvloc (uact, addnlcc, dvlocin)
+  ELSE
+    ! Bring potential in reciprocal space
+    !$acc host_data use_device(dvlocin)
+    CALL fwfft ('Rho', dvlocin, dffts)
+    !$acc end host_data
+    IF (gg(1) < 1d-8) THEN
+      !$acc kernels
+      dvlocin(dffts%nl(1)) = 1d0
+     !$acc end kernels
+    ENDIF
+    ! Bring potential in real space
+   !$acc host_data use_device(dvlocin)
+    CALL invfft ('Rho', dvlocin, dffts)
+   !$acc end host_data
+  ENDIF
   ! Now we compute dV_loc/dtau * psi in real space
   !
   !$acc update device(evc)
@@ -164,7 +182,11 @@ subroutine dvqpsi_us (ik, uact, addnlcc, becp1, alphap)
   !   First a term similar to the KB case.
   !   Then a term due to the change of the D coefficients.
   !
-  call dvqpsi_us_only (ik, uact, becp1, alphap)
+  IF (.NOT. lmultipole) call dvqpsi_us_only (ik, uact, becp1, alphap)
+  !
+  ! DFPT+U: add the bare variation of the Hubbard potential
+  !
+  IF (lda_plus_u) CALL dvqhub_barepsi_us(ik, uact)
   !
   call stop_clock_gpu ('dvqpsi_us')
   !
