@@ -8,7 +8,7 @@
 !----------------------------------------------------------------------------
 SUBROUTINE iosys()
   !-----------------------------------------------------------------------------
-  !! Copy data read from input file (subroutine \(\texttt{read_input_file}\)
+  !! Copy data read from input file (subroutine \(\tex/temttt{read_input_file}\)
   !! and stored in modules input_parameters into internal modules.  
   !! Wrapper routine: the original "iosys" was too long and was split into
   !! more *iosys* routines, containing input_parameters module
@@ -287,9 +287,9 @@ SUBROUTINE control_iosys()
  
   !
   USE extrapolation, ONLY : pot_order, wfc_order
-  USE control_flags, ONLY : isolve, max_cg_iter, max_ppcg_iter, david, &
+  USE control_flags, ONLY : isolve, max_cg_iter, david, &
                             rmm_ndim, rmm_conv, gs_nblock, rmm_with_davidson, &
-                            tr2, imix, gamma_only, &
+                            tr2, imix, gamma_only, tnosep, tnoseh, &
                             nmix, iverbosity, smallmem, nexxiter, niter, &
                             io_level, ethr, lscf, lbfgs, lmd, &
                             lbands, lconstrain, restart, &
@@ -396,7 +396,7 @@ SUBROUTINE control_iosys()
                                mixing_ndim, mixing_fixed_ns, conv_thr,     &
                                tqr, tq_smoothing, tbeta_smoothing,         &
                                diago_thr_init,                             &
-                               diago_cg_maxiter, diago_ppcg_maxiter,       &
+                               diago_cg_maxiter,                           &
                                diago_david_ndim, diago_rmm_ndim,           &
                                diago_rmm_conv, diago_gs_nblock,            &
                                diagonalization, diago_full_acc,            &
@@ -414,6 +414,11 @@ SUBROUTINE control_iosys()
                                trust_radius_ini, bfgs_ndim, tgdiis_step,       &
                                fire_nmin, fire_f_inc, fire_f_dec, &
                                fire_alpha_init, fire_falpha, fire_dtmax
+!
+! ... IONS NOSE HOOVER THERMOSTAT
+
+  USE input_parameters, ONLY: fnosep, nhpcl, nhptyp, ndega, nhgrp, fnhscl  
+                           
   !
   ! ... CELL namelist
   !
@@ -421,6 +426,10 @@ SUBROUTINE control_iosys()
                                cell_temperature, cell_factor, press_conv_thr, &
                                cell_dofree, treinit_gvecs 
   !
+  ! ... CELL NOSE HOOVER THERMOSTAT
+  ! 
+  USE input_parameters, ONLY: fnoseh, temph                                      
+  !
   ! ... WANNIER_NEW namelist
   !
   USE input_parameters, ONLY : use_wannier, nwan, use_energy_int, print_wannier_coeff
@@ -530,6 +539,10 @@ SUBROUTINE control_iosys()
         !
         calc        = 'vd'
         !
+     CASE ('velocity-verlet')
+        !
+        calc = 'wd'
+        !
      CASE( 'langevin' )
         !
         calc        = 'ld'
@@ -951,12 +964,12 @@ SUBROUTINE control_iosys()
      isolve = 1
      max_cg_iter = diago_cg_maxiter
      !
-  CASE ( 'ppcg' )
+  CASE ( 'ppcg', 'PPCG' )
      !
      isolve = 2
-     max_ppcg_iter = diago_ppcg_maxiter
+     CALL errore( 'iosys', 'PPCG diagonalization not supported anymore (Dec. 2024)', 1 )
      !
-  CASE ( 'paro' )
+  CASE ( 'paro', 'ParO' )
      !
      isolve = 3
      !
@@ -1119,6 +1132,13 @@ SUBROUTINE control_iosys()
      temperature  = tempw
      nraise_      = nraise
      !
+  CASE ('nose')
+     thermostat = trim(ion_temperature)
+     temperature = tempw 
+     tnosep = .true. 
+     control_temp = .true.
+
+
   CASE DEFAULT
      !
      CALL errore( 'iosys', &
@@ -1126,6 +1146,18 @@ SUBROUTINE control_iosys()
      !
   END SELECT
   !
+  ! CELL TEMPERATURE
+  ! 
+  SELECT CASE (trim(cell_temperature))
+    CASE ('nose')
+      IF (.not. (control_temp .or. tnosep)) temperature = temph 
+      tnoseh = .true.
+    CASE ('not_controlled', 'not-controlled', 'not controlled') 
+      tnoseh = .false. 
+    CASE DEFAULT 
+      CALL errore('iosys', 'unsupported cell_temperature',1)
+  END SELECT 
+  !
   ! SELF-CONSISTENCY
   !
   SELECT CASE( trim( mixing_mode ) )
@@ -1222,7 +1254,7 @@ SUBROUTINE control_iosys()
   !
   ! DFT+U
   !
-  CALL dftu_iosys ( nsp, noncolin )
+  CALL dftu_iosys ( nsp )
   !
   ! REAL-SPACE TREATMENT
   !
@@ -1313,6 +1345,8 @@ SUBROUTINE control_iosys()
   IF ( llondon.AND.lxdm .OR. llondon.AND.ts_vdw_ .OR. lxdm.AND.ts_vdw_ .OR. &
            ldftd3.AND.llondon .OR. ldftd3.AND.lxdm .OR. ldftd3.AND.ts_vdw ) &
      CALL errore("iosys","must choose a unique vdW correction!", 1)
+  IF ( lxdm .AND. noncolin ) &
+     CALL errore("iosys","XDM not implemented for noncolinear case", 1)
   !
   IF ( llondon) THEN
      lon_rcut    = london_rcut
@@ -1848,13 +1882,16 @@ SUBROUTINE pos_iosys ( )
   USE input_parameters,   ONLY : taspc, tapos, rd_pos, atomic_positions,  &
                                  rd_if_pos, ibrav, nat_ => nat, ntyp,     &
                                  sp_pos, rd_for, tavel, sp_vel, rd_vel,   &
-                                 atom_mass, atom_label, lsg
+                                 atom_mass, atom_label, lsg, tempw,&
+                                 fnosep, nhpcl, nhptyp, ndega, nhgrp, fnhscl
   USE kinds,              ONLY : DP
-  USE dynamics_module,    ONLY : vel
+  USE dynamics_module,    ONLY : vel, get_ndof
   USE force_mod,          ONLY : force
   USE ions_base,          ONLY : nat, nsp, ityp, tau, atm, &
-                                 extfor, if_pos, amass, fixatom, tau_format
-  USE control_flags,      ONLY : textfor, tv0rd
+                                 extfor, if_pos, amass, fixatom, tau_format, &
+                                 tions_base_init
+  USE ions_nose,          ONLY:  ions_nose_init 
+  USE control_flags,      ONLY : textfor, tv0rd, tnosep
   USE wyckoff,            ONLY : nattot, tautot, ityptot, extfortot, &
                                  if_postot, clean_spacegroup
   !
@@ -1944,11 +1981,16 @@ SUBROUTINE pos_iosys ( )
   !
   tau_format = trim( atomic_positions )
   CALL convert_tau ( tau_format, nat, tau )
+  IF (tnosep) THEN 
+     tions_base_init = .TRUE. 
+     IF (ndega == 0) ndega = get_ndof() 
+     call ions_nose_init(tempw, fnosep, nhpcl, nhptyp, ndega, nhgrp, fnhscl)
+  END IF 
   !
 END SUBROUTINE pos_iosys
 !
 !----------------------------------------------------------------------------
-SUBROUTINE dftu_iosys ( ntyp, noncolin )
+SUBROUTINE dftu_iosys ( ntyp )
   !----------------------------------------------------------------------------
   !
   ! Hubbard parameters: input
@@ -1960,18 +2002,23 @@ SUBROUTINE dftu_iosys ( ntyp, noncolin )
                                Hubbard_U, Hubbard_J, Hubbard_J0, Hubbard_V, Hubbard_U2, &
                                Hubbard_n, Hubbard_l, Hubbard_projectors, &
                                Hubbard_n2, Hubbard_l2, Hubbard_n3, Hubbard_l3, &
-                               lda_plus_u, lda_plus_u_kind
+                               lda_plus_u, lda_plus_u_kind, Hubbard_Um, Hubbard_alpha_m, &
+                               Hubbard_Um_nc, Hubbard_alpha_m_nc
   !
   USE constants,     ONLY : rytoev
   !
   ! Hubbard parameters: output
   !
   USE ldaU,          ONLY : Hubbard_U_  => hubbard_u, &
+                            Hubbard_Um_  => hubbard_um, &
+                            Hubbard_Um_nc_  => hubbard_um_nc, &
                             Hubbard_J0_ => hubbard_j0, &
                             Hubbard_J_ => hubbard_j, &
                             Hubbard_n_ => hubbard_n, &
                             Hubbard_l_ => hubbard_l, &
                             Hubbard_alpha_ => hubbard_alpha, &
+                            Hubbard_alpha_m_ => hubbard_alpha_m, &
+                            Hubbard_alpha_m_nc_ => hubbard_alpha_m_nc, &
                             Hubbard_beta_ => hubbard_beta, &
                             lda_plus_u_    => lda_plus_u, &
                             lda_plus_u_kind_    => lda_plus_u_kind, &
@@ -1995,7 +2042,6 @@ SUBROUTINE dftu_iosys ( ntyp, noncolin )
   !
   ! Needed input variables
   INTEGER, INTENT(IN) :: ntyp
-  LOGICAL, INTENT(IN) :: noncolin
   !
   !
   lda_plus_u_      = lda_plus_u
@@ -2003,6 +2049,8 @@ SUBROUTINE dftu_iosys ( ntyp, noncolin )
   !
   !
   Hubbard_U_(1:ntyp)          = hubbard_u(1:ntyp) / rytoev
+  Hubbard_Um_(:,:,:)     = hubbard_um(:,:,:) / rytoev
+  Hubbard_Um_nc_(:,:)    = hubbard_um_nc(:,:) / rytoev
   Hubbard_J_(1:3,1:ntyp)      = hubbard_j(1:3,1:ntyp) / rytoev
   Hubbard_J0_(1:ntyp)         = hubbard_j0(1:ntyp) / rytoev
   Hubbard_V_(:,:,:)           = hubbard_V(:,:,:) / rytoev
@@ -2012,9 +2060,11 @@ SUBROUTINE dftu_iosys ( ntyp, noncolin )
   Hubbard_n2_(1:ntyp)         = hubbard_n2(1:ntyp)
   Hubbard_l2_(1:ntyp)         = hubbard_l2(1:ntyp)
   Hubbard_n3_(1:ntyp)         = hubbard_n3(1:ntyp)
-  Hubbard_l3_(1:ntyp)         = hubbard_l3(1:ntyp)
+  Hubbard_l3_(1:ntyp)         = hubbard_l3(1:ntyp) 
   Hubbard_projectors_         = hubbard_projectors
-  Hubbard_alpha_(1:ntyp)      = hubbard_alpha(1:ntyp) / rytoev
+  Hubbard_alpha_(:)      = hubbard_alpha(:) / rytoev
+  Hubbard_alpha_m_(:,:,:) = hubbard_alpha_m(:,:,:) / rytoev
+  Hubbard_alpha_m_nc_(:,:) = hubbard_alpha_m_nc(:,:) / rytoev
   Hubbard_beta_(1:ntyp)       = hubbard_beta(1:ntyp) / rytoev
   Hubbard_occ_(1:ntyp,1:3)    = hubbard_occ(1:ntyp,1:3)
   Hubbard_alpha_back_(1:ntyp) = hubbard_alpha_back(1:ntyp) / rytoev
@@ -2133,7 +2183,10 @@ SUBROUTINE set_wmass ( )
   USE constants,     ONLY : pi
   USE cell_base,     ONLY : wmass, omega
   USE ions_base,     ONLY : ityp, nat, amass
+  USE cell_nose,     ONLY : cell_nose_init
+  USE input_parameters, ONLY: fnoseh, temph
   USE cellmd,        ONLY : calc, lmovecell
+  USE control_flags, ONLY : tnoseh
   !
   IMPLICIT NONE
   INTEGER :: ia
@@ -2157,5 +2210,6 @@ SUBROUTINE set_wmass ( )
   ENDIF
   IF ( wmass <= 0.D0 ) CALL errore( 'set_wmass', &
             & 'vcsmd: a positive value for cell mass is required', 1 )
+  IF (tnoseh) CALL cell_nose_init(temph, fnoseh)
   !
 END SUBROUTINE set_wmass
