@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2022 Quantum ESPRESSO group
+! Copyright (C) 2001-2025 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -38,6 +38,10 @@ SUBROUTINE new_ns( ns )
   USE becmod,               ONLY : bec_type, calbec, &
                                    allocate_bec_type, deallocate_bec_type
   USE noncollin_module,     ONLY : colin_mag
+#if defined (__OSCDFT)
+  USE plugin_flags,         ONLY : use_oscdft
+  USE oscdft_base,          ONLY : oscdft_ctx
+#endif  
   !
   IMPLICIT NONE
   !
@@ -135,6 +139,17 @@ SUBROUTINE new_ns( ns )
      ENDDO
   ENDDO
   !
+#if defined (__OSCDFT)
+  IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==2) .AND. &
+                  .NOT.oscdft_ctx%inp%constraint_diag) THEN
+     IF (.NOT.oscdft_ctx%conv) THEN
+        ns(:,:,:,:) = nr(:,:,:,:)
+        DEALLOCATE(nr)
+        RETURN
+     ENDIF
+  ENDIF
+#endif
+  !
   ! symmetrize the quantities nr -> ns
   !
   ns (:,:,:,:) = 0.d0
