@@ -34,9 +34,6 @@ SUBROUTINE write_casino_wfn(gather,blip,multiplicity,binwrite,single_precision_b
    USE mp_bands, ONLY: intra_bgrp_comm
    USE mp, ONLY: mp_sum, mp_gather, mp_bcast, mp_get
    USE buffers,            ONLY : get_buffer
-   USE wavefunctions_gpum, ONLY : using_evc
-   USE wvfct_gpum,         ONLY : using_et
-
    USE pw2blip
 
    IMPLICIT NONE
@@ -81,8 +78,6 @@ SUBROUTINE write_casino_wfn(gather,blip,multiplicity,binwrite,single_precision_b
    INTEGER,SAVE :: ran_array_idx=-1
    REAL(DP),SAVE :: ran_array(Nran)
 
-   CALL using_evc(0)
-
    dowrite=ionode.or..not.(gather.or.blip)
 
    ALLOCATE (idx (ngm) )
@@ -212,14 +207,11 @@ SUBROUTINE write_casino_wfn(gather,blip,multiplicity,binwrite,single_precision_b
    iorb = 0
    norb = nk*nspin*nbnd
 
-   CALL using_evc(0)
-
    DO ik = 1, nk
       DO ispin = 1, nspin
          ikk = ik + nk*(ispin-1)
          npw = ngk(ikk)
          IF( nks > 1 ) CALL get_buffer(evc,nwordwfc,iunwfc,ikk)
-         IF( nks > 1 ) CALL using_evc(2)
          DO ibnd = 1, nbnd
             evc_l(:) = (0.d0, 0d0)
             evc_l(gtoig(igk_k(1:npw,ikk))) = evc(1:npw,ibnd)
@@ -366,13 +358,10 @@ CONTAINS
             ENDDO
          ENDDO
 
-         CALL using_evc(0); CALL using_et(0)
-
          DO ik = 1, nk
             ikk = ik + nk*(ispin-1)
             npw = ngk(ikk)
             IF( nks > 1 ) CALL get_buffer (evc, nwordwfc, iunwfc, ikk )
-            IF( nks > 1 ) CALL using_evc(2)
             !
             CALL init_us_2 (npw, igk_k(1,ikk), xk (1, ikk), vkb)
             CALL calbec ( npw, vkb, evc, becp )
@@ -720,7 +709,6 @@ CONTAINS
          kprod(6,:)=kvec(2,:)*kvec(3,:)
          ksq(:)=kprod(1,:)+kprod(2,:)+kprod(3,:)
 
-         CALL using_et(0)
          WRITE(iob)&
             kvec                                          ,&
             ksq                                           ,&
@@ -875,8 +863,6 @@ CONTAINS
 
       IF(binwrite)RETURN
 
-      CALL using_et(0)
-
       ikk = ik + nk*(ispin-1)
       IF(ispin==1.and.ibnd==1)THEN
          WRITE(io,'(a)') ' k-point # ; # of bands (up spin/down spin); &
@@ -903,8 +889,6 @@ CONTAINS
       INTEGER,INTENT(in) :: ik,ispin,ibnd
       INTEGER lx,ly,lz,ikk,j,l1,l2,l3
 
-      CALL using_et(0)
-
       IF(binwrite)THEN
          DO l3=1,blipgrid(3)
             DO l2=1,blipgrid(2)
@@ -950,8 +934,6 @@ CONTAINS
       INTEGER,INTENT(in) :: ik,ispin,ibnd,re_im
       INTEGER lx,ly,lz,ikk,j,l1,l2,l3
 
-      CALL using_et(0)
-
       IF(binwrite)THEN
          IF(re_im==1)THEN
             DO l3=1,blipgrid(3)
