@@ -104,16 +104,16 @@ MODULE pw_restart_new
       USE fixed_occ,            ONLY : tfixed_occ, f_inp
       USE ktetra,               ONLY : tetra_type
       USE ldaU,                 ONLY : lda_plus_u, lda_plus_u_kind, Hubbard_projectors, &
-                                       Hubbard_lmax, Hubbard_l, Hubbard_n, Hubbard_U, Hubbard_J, &
-                                       Hubbard_n2, Hubbard_n3, Hubbard_l2, Hubbard_l3, Hubbard_V, Hubbard_occ,&
-                                       Hubbard_alpha, Hubbard_alpha_back, nsg, &
-                                       Hubbard_J0, Hubbard_beta, Hubbard_U2, ityp_s, &
-                                       is_hubbard, is_hubbard_back, backall, neighood, nsg
+                                       Hubbard_lmax, Hubbard_l, Hubbard_n, Hubbard_U, Hubbard_Um, Hubbard_Um_nc, & 
+                                       Hubbard_J, Hubbard_n2, Hubbard_n3, Hubbard_l2, Hubbard_l3, Hubbard_V,     & 
+                                       Hubbard_occ, Hubbard_alpha, Hubbard_alpha_back, nsg, order_um, Hubbard_J0,&
+                                       Hubbard_beta, Hubbard_U2, is_hubbard, is_hubbard_back, backall, neighood, &
+                                       nsg
       USE symm_base,            ONLY : nrot, nsym, invsym, s, ft, irt, &
                                        t_rev, sname, time_reversal, no_t_rev,&
                                        spacegroup
       USE lsda_mod,             ONLY : nspin, isk, lsda, starting_magnetization, magtot, & 
-                                       absmag, local_charges, local_mag
+                                       absmag, local_charges, local_mag 
       USE noncollin_module,     ONLY : angle1, angle2, i_cons, mcons, bfield, &
                                        magtot_nc, lambda, domag, lspinorb
       USE funct,                ONLY : get_dft_short, get_nonlocc_name, dft_is_nonlocc
@@ -237,7 +237,7 @@ MODULE pw_restart_new
       ! If not allocated, there is no such variable
       !
       REAL(DP), ALLOCATABLE :: london_c6_(:), bp_el_pol(:), bp_ion_pol(:), &
-           U_opt(:), J0_opt(:), alpha_opt(:), J_opt(:,:), beta_opt(:), &
+           U_opt(:), Um_opt(:,:,:), J0_opt(:), alpha_opt(:), J_opt(:,:), beta_opt(:), &
            U2_opt(:), alpha_back_opt(:), ef_updw(:), nsg_(:,:,:,:)
       INTEGER,ALLOCATABLE :: n_opt(:), l_opt(:), l2_opt(:), l3_opt(:), n2_opt(:), n3_opt(:)
       LOGICAL, ALLOCATABLE :: backall_opt(:) 
@@ -506,8 +506,16 @@ MODULE pw_restart_new
             CALL check_and_allocate_logical(backall_opt, backall)
             IF ( ANY(Hubbard_J(:,1:nsp) /= 0.0_DP)) THEN
                ALLOCATE (J_opt(3,nsp)) 
-               J_opt(:, 1:nsp) = Hubbard_J(:, 1:nsp) 
+               J_opt(:, 1:nsp) = Hubbard_J(:, 1:nsp)  
             END IF
+            IF (ANY(Hubbard_Um(1:2*Hubbard_lmax+1,1:MIN(nspin,2),1:nsp)/=0.0_dp)) THEN
+              ALLOCATE (Um_opt(2*Hubbard_lmax+1,MIN(nspin,2),nsp))
+              Um_opt(1:2*Hubbard_lmax+1,1:MIN(nspin,2),1:nsp) = &
+                      Hubbard_Um(1:2*Hubbard_lmax+1,1:MIN(nspin,2),1:nsp) * Ry_to_Ha  
+            ELSE IF (ANY(Hubbard_Um_nc(1:4*Hubbard_lmax+2,1:nsp)/=0.0_dp)) THEN 
+              ALLOCATE(Um_opt(4*Hubbard_lmax+2,1,1:nsp))
+              Um_opt(1:4*Hubbard_lmax+2,1,1:nsp) = Hubbard_Um_nc(1:4*Hubbard_lmax+2,1:nsp) * Ry_to_Ha 
+            END IF 
             IF (lda_plus_u_kind==2) THEN
                ALLOCATE (nsg_(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
                nsg_ = 0.0d0
@@ -536,9 +544,10 @@ MODULE pw_restart_new
                     ITYP = ityp(1:nat), IS_HUBBARD = is_hubbard, IS_HUBBARD_BACK = is_hubbard_back, BACKALL = backall,& 
                     HUBB_OCC = Hubbard_occ, HUBB_n2 = n2_opt, HUBB_L2 = l2_opt, HUBB_L3 = l3_opt, NONCOLIN = noncolin,& 
                     HUBB_N3 = n3_opt, LDA_PLUS_U_KIND = lda_plus_u_kind, U_PROJECTION_TYPE = Hubbard_projectors,      &
-                    U =U_opt, U2 = U2_opt, J0 = J0_opt, J = J_opt, n = n_opt, l = l_opt,                              &
+                    U =U_opt, Um = Um_opt, U2 = U2_opt, J0 = J0_opt, J = J_opt, n = n_opt, l = l_opt,                 &
                     Hubbard_V = Hubbard_V *Ry_to_Ha, alpha = alpha_opt, beta = beta_opt, alpha_back = alpha_back_opt, & 
-                    starting_ns = starting_ns_eigenvalue, Hub_ns = rho%ns, Hub_ns_nc = rho%ns_nc, Hub_nsg = nsg_)
+                    starting_ns = starting_ns_eigenvalue, Hub_ns = rho%ns, Hub_ns_nc = rho%ns_nc, order_um = order_um, & 
+                    Hub_nsg = nsg_)
             !
             IF (ALLOCATED(J_opt)) DEALLOCATE(J_opt)
             IF (ALLOCATED(nsg_))  DEALLOCATE(nsg_)
@@ -1167,7 +1176,8 @@ MODULE pw_restart_new
       USE ldaU,            ONLY : lda_plus_u, lda_plus_u_kind, Hubbard_lmax, Hubbard_lmax_back, &
                                   Hubbard_n, Hubbard_l, Hubbard_n2, Hubbard_l2, Hubbard_n3, Hubbard_l3, backall, &
                                   Hubbard_U, Hubbard_U2, Hubbard_J, Hubbard_V, Hubbard_alpha, Hubbard_occ, &
-                                  Hubbard_alpha_back, Hubbard_J0, Hubbard_beta, Hubbard_projectors
+                                  Hubbard_alpha_back, Hubbard_J0, Hubbard_beta, Hubbard_projectors, Hubbard_Um, & 
+                                  Hubbard_Um_nc, apply_u, Hubbard_alpha_m, Hubbard_alpha_m_nc
       USE funct,           ONLY : enforce_input_dft, get_dft_short
       USE xc_lib,          ONLY : start_exx, exx_is_active,xclib_dft_is,      &
                                   set_screening_parameter, set_gau_parameter, &
@@ -1216,7 +1226,7 @@ MODULE pw_restart_new
       LOGICAL  :: magnetic_sym, lvalid_input
       CHARACTER(LEN=37)  :: dft_name
       CHARACTER(LEN=256) ::dft_
-      INTEGER           :: npwx_g
+      INTEGER           :: npwx_g, llmax, ntmax
       CHARACTER(LEN=320):: filename
       REAL(dp) :: exx_fraction, screening_parameter
       TYPE (output_type)        :: output_obj 
@@ -1287,14 +1297,26 @@ MODULE pw_restart_new
       CALL qexsd_copy_dft ( output_obj%dft, nsp, atm, &
            dft_name, nq1, nq2, nq3, ecutfock, exx_fraction, screening_parameter, &
            exxdiv_treatment, x_gamma_extrapolation, ecutvcut, local_thr, &
-           lda_plus_u, lda_plus_u_kind, Hubbard_projectors, Hubbard_n, Hubbard_l, Hubbard_lmax, Hubbard_occ,&
+           lda_plus_u, apply_u,lda_plus_u_kind, Hubbard_projectors, Hubbard_n, Hubbard_l, Hubbard_lmax, Hubbard_occ,&
            Hubbard_n2, Hubbard_l2, Hubbard_n3, Hubbard_l3, backall, Hubbard_lmax_back, Hubbard_alpha_back, &
-           Hubbard_U, Hubbard_U2, Hubbard_J0, Hubbard_alpha, Hubbard_beta, Hubbard_J, Hubbard_V, &
+           Hubbard_U, Hubbard_Um, Hubbard_U2, Hubbard_J0, Hubbard_alpha, Hubbard_alpha_m, Hubbard_beta, Hubbard_J, Hubbard_V, &
            vdw_corr, dftd3_version, dftd3_threebody, scal6, lon_rcut, vdw_isolated )
       Hubbard_alpha_back = Hubbard_alpha_back * e2 
       Hubbard_alpha      = Hubbard_alpha      * e2
       Hubbard_beta       = Hubbard_beta       * e2 
       Hubbard_U          = Hubbard_U          * e2 
+      Hubbard_Um         = Hubbard_Um         * e2
+      Hubbard_Um_nc      = 0.0_DP 
+      Hubbard_alpha_m   = Hubbard_alpha_m     * e2 
+      Hubbard_alpha_m_nc = 0.0_DP 
+      IF (noncolin) THEN
+        llmax = SIZE(Hubbard_Um_nc,1)/2   
+        ntmax = SIZE(Hubbard_Um_nc,2) 
+        Hubbard_Um_nc(:,:) = RESHAPE(Hubbard_Um(:,:,:),[2*llmax,ntmax]) 
+        Hubbard_alpha_m_nc(:,:) = RESHAPE(Hubbard_alpha_m(:,:,:),[2*llmax, ntmax])   
+        Hubbard_Um = 0.0_DP 
+        Hubbard_alpha_m = 0.0_DP 
+      END IF
       Hubbard_U2         = Hubbard_U2         * e2 
       Hubbard_V          = Hubbard_V          * e2 
       Hubbard_J0         = Hubbard_J0         * e2 
