@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2008-2010 Quantum ESPRESSO group
+! Copyright (C) 2008-2025 Quantum ESPRESSO Foundation
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -540,6 +540,10 @@ CONTAINS
     ! some gcut_ value (see above) cuts a shell of G-vectors in the middle
     ! This may happen if G-vector ordering with |G| is not perfect
     !
+    ierr = 0
+10  ierr = ierr+1
+    IF ( ierr > 5 ) CALL errore('sym_rho_init', &
+         'internal error: G-vector distribution failed', ierr)
     ngpos=0
     gtop = 0.0_dp
     gnext= 0.0_dp
@@ -547,7 +551,12 @@ CONTAINS
        ngloc=0
 cutg:  DO ig=ngpos+1,ngm
           IF ( gg(ig) > gcut_(np) ) THEN
-             gtop = gg(ig-1)
+             ! this is to prevent an unlikely out-of-bound error
+             IF ( ig > 1 ) THEN
+                gtop = gg(ig-1)
+             ELSE
+                gtop = gcut_(np)
+             END IF
              gnext = gg(ig)
              EXIT cutg
           END IF
@@ -557,13 +566,19 @@ cutg:  DO ig=ngpos+1,ngm
             'some processors have no G-vectors for symmetrization')
        ngpos = ngpos + ngloc
        IF ( ngpos > ngm ) &
-            CALL errore('sym_rho','internal error: too many G-vectors', ngpos)
+            CALL errore('sym_rho_init','internal error : too many G-vectors', ngpos)
        ! Note that gnext > gtop only for perfect ordering
        CALL mp_max( gtop , intra_bgrp_comm)
        CALL mp_min( gnext, intra_bgrp_comm)
-       ! The following criterion is rather arbitrary: it should as small as
-       ! possible but slightly larger than the expected numerical noise 
-       IF ( ABS ( gnext-gtop ) < 1.0e-7*gcut_(np) ) gcut_(np) = MAX(gnext,gtop)
+       ! The following criterion is rather arbitrary:
+       ! the quantity at the rhs below should be as small as possible,
+       ! but larger than the expected numerical noise
+       IF ( ABS ( gnext-gtop ) < 1.0e-7*gcut_(np) ) THEN
+          ! The largest vector in this slice is too close to the smallest one
+          ! in the next slice: raise the gcut_ for this slice, check again
+          gcut_(np) = MAX(gnext,gtop)
+          GO TO 10
+       END IF
     END DO
     !
     ! now find the number of G-vectors in each "slice"
@@ -735,7 +750,7 @@ gloop:    DO jg=iig,ngm_
   END SUBROUTINE sym_rho_init_shells
   !
   !-----------------------------------------------------------------------
-  SUBROUTINE sym_rho (nspin, rhog)
+  SUBROUTINE sym_rho (nspin, rhog, is_sym )
     !-----------------------------------------------------------------------
     !! Symmetrize the charge density rho in reciprocal space.
     !
@@ -755,6 +770,8 @@ gloop:    DO jg=iig,ngm_
     COMPLEX(DP), INTENT(INOUT) :: rhog(ngm,nspin)
     !! components of rho: rhog(ig) = rho(G(:,ig)). 
     !! Unsymmetrized on input, symmetrized on output
+    INTEGER, INTENT(IN), OPTIONAL :: is_sym( nsym )
+    !! symmetry operations used for symmetrization (1 = yes, 0 = no)
     !
     ! ... local variables
     !
@@ -766,7 +783,7 @@ gloop:    DO jg=iig,ngm_
     IF ( no_rho_sym) RETURN
 #if !defined(__MPI)
     !
-    CALL sym_rho_serial ( ngm, g, nspin, rhog )
+    CALL sym_rho_serial ( ngm, g, nspin, rhog, is_sym )
     !
 #else
     !
@@ -790,7 +807,7 @@ gloop:    DO jg=iig,ngm_
     !
     !   Now symmetrize
     !
-    CALL sym_rho_serial ( ngm_, g_, nspin, rhog_ )
+    CALL sym_rho_serial ( ngm_, g_, nspin, rhog_, is_sym )
     !
     DEALLOCATE ( g_ )
     !
@@ -812,7 +829,7 @@ gloop:    DO jg=iig,ngm_
   END SUBROUTINE sym_rho
   !
   !-----------------------------------------------------------------------
-  SUBROUTINE sym_rho_serial ( ngm_, g_, nspin_, rhog_ )
+  SUBROUTINE sym_rho_serial ( ngm_, g_, nspin_, rhog_, is_sym )
     !-----------------------------------------------------------------------
     !! Symmetrize the charge density rho in reciprocal space.    
     !
@@ -832,6 +849,8 @@ gloop:    DO jg=iig,ngm_
     COMPLEX(DP), INTENT(INOUT) :: rhog_( ngm_, nspin_ )
     !! rho in reciprocal space: rhog_(ig) = rho(G(:,ig)). 
     !! Unsymmetrized on input, symmetrized on output
+    INTEGER, INTENT(IN), OPTIONAL :: is_sym( nsym )
+    !! symmetry operations used for symmetrization (1 = yes, 0 = no)
     !
     ! ... local variables
     !
@@ -841,10 +860,19 @@ gloop:    DO jg=iig,ngm_
     INTEGER :: irot(48), ig, isg, igl, ng, ns, nspin_lsda, is
     LOGICAL, ALLOCATABLE :: done(:)
     LOGICAL :: non_symmorphic(48)
+    INTEGER :: nsym_used, use_sym(nsym)
+    !
+    IF ( PRESENT( is_sym ) ) THEN
+       use_sym(1:nsym) = is_sym(1:nsym)
+    ELSE
+       use_sym(1:nsym) = 1
+    END IF
+    nsym_used = SUM ( use_sym(1:nsym) )
     !
     ! convert fractional translations to cartesian, in a0 units
     !
     DO ns=1,nsym
+       IF ( use_sym(ns) == 0 ) CYCLE
        non_symmorphic(ns) = ( ft(1,ns) /= 0.0_dp .OR. &
                               ft(2,ns) /= 0.0_dp .OR. &
                               ft(3,ns) /= 0.0_dp )
@@ -855,7 +883,6 @@ gloop:    DO jg=iig,ngm_
     !
     IF ( nspin_ == 4 ) THEN
        nspin_lsda = 1
-       !
     ELSE IF ( nspin_ == 1 .OR. nspin_ == 2 ) THEN
        nspin_lsda = nspin_
     ELSE
@@ -888,7 +915,7 @@ gloop:    DO jg=iig,ngm_
              magsum(:) = (0.0_dp, 0.0_dp)
              ! S^{-1} are needed here
              DO ns=1,nsym
-
+                IF ( use_sym(ns) == 0 ) CYCLE
                 sg(:) = s(:,1,invs(ns)) * g0(1,ig) + &
                         s(:,2,invs(ns)) * g0(2,ig) + &
                         s(:,3,invs(ns)) * g0(3,ig)
@@ -958,13 +985,14 @@ gloop:    DO jg=iig,ngm_
              END DO
              !
              DO is=1,nspin_lsda
-                rhosum(is) = rhosum(is) / nsym
+                rhosum(is) = rhosum(is) / nsym_used
              END DO
-             IF ( nspin_ == 4 ) magsum(:) = magsum(:) / nsym
+             IF ( nspin_ == 4 ) magsum(:) = magsum(:) / nsym_used
              !
              !  now fill the shell of G-vectors with the symmetrized value
              !
              DO ns=1,nsym
+                IF ( use_sym(ns) == 0 ) CYCLE
                 isg = shell(igl)%vect(irot(ns))
                 IF ( nspin_ == 4 ) THEN
                    ! rotate magnetization
