@@ -46,9 +46,7 @@ SUBROUTINE lr_dealloc()
   IF (allocated(sevc1))     DEALLOCATE(sevc1)
   IF (allocated(d0psi))     DEALLOCATE(d0psi)
   IF (allocated(d0psi2))    DEALLOCATE(d0psi2)
-  IF (allocated(tg_revc0))  DEALLOCATE(tg_revc0)
   IF (allocated(tg_psic))   DEALLOCATE(tg_psic)
-  IF (allocated(revc0))     DEALLOCATE(revc0)
   IF (allocated(bbg))       DEALLOCATE(bbg)
   IF (allocated(bbk))       DEALLOCATE(bbk)
   IF (allocated(bbnc))      DEALLOCATE(bbnc)
